#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import os
import sys
import re
import pwd
import subprocess

if sys.version_info < (3,):
    bytes = str
    PY3 = False
else:
    unicode = str
    PY3 = True

#add lib path
playbook_file = os.path.realpath(sys.argv[0])
playbook_path=os.path.split(playbook_file)[0]   

lib_path=os.path.join(playbook_path,'library')
if os.path.exists(lib_path) is True:
    sys.path.insert(0,lib_path)
    lib_path=os.path.join(lib_path,'securecat')
    if os.path.exists(lib_path) is True:
        sys.path.insert(0,lib_path)
        modulepath = os.environ.get('CSC_MODULE_PATH')
        if modulepath is None:
            os.environ['CSC_MODULE_PATH'] = lib_path
    
from utils import playbook
from linux.mysql_conf_parser import MySQLParse
_find_unsafe = re.compile(r'[^\w@%+=:,./-]').search

def add_escape(value):
    reserved_chars = r'''$'"'''
    replace = ['\\' + l for l in reserved_chars]
    trans = str.maketrans(dict(zip(reserved_chars, replace)))
    return value.translate(trans)


def quote(s):
    """Return a shell-escaped version of the string *s*."""
    if not s:
        return "''"
    if _find_unsafe(s) is None:
        return s

    # use single quotes, and put single quotes into double quotes
    # the string $'b is then quoted as '$'"'"'b'
    return "'" + add_escape(s) + "'"


class MyPlaybook(playbook.Playbook):
    
    def set_extra_vars_pre(self):
        self._runcontext.set_extra_var('MYSQL_PID','')
        self._runcontext.set_extra_var('MYSQL_PORT','3306')
    
    def get_user_name(self, uid):
        users = pwd.getpwall()
        for user in users:
            if user.pw_uid == uid:
                return user.pw_name
        return '%s' % uid
    
    def set_extra_vars_post(self):
        '''
        处理流程
        1：通过MYSQL_PID，找到父进程的，获取父进程的用户，MYSQL_PARENT_USER，父进程的启动命令行
        
        2：如果父进程命令行包含mysqld_multi，说明是通过多实例方式启动的，所有配置参数会在MySQL启动命令行中包括

        3：如果没有找到mysqld_multi，说明是单实例启动
        1）：如果找到--defaults-file=<file>，则从配置文件中读取配置
        2）：如果没有，则按照下面的顺序依次查找配置文件
            /etc/my.cnf Global options 
            /etc/mysql/my.cnf Global options 
            $MYSQL_BASEDIR/my.cnf Server-specific options (server only) 
            defaults-extra-file The file specified with --defaults-extra-file, if any 
            ~/.my.cnf User-specific options 
            ~/.mylogin.cnf User-specific login path options (clients only) 
            
            ~ 表示父进程用户的Home目录
        4：通过mysqld --verbose --help 获取当前实例basedir、datadir、plugin-dir、tmpdir
        '''
        
        var_mysql_pid = self._runcontext.get_extra_var('PID')
        
        if var_mysql_pid == '':
            raise Exception('Must specify extra variable MYSQL_PID.')
        
        #解析默认配置
        #获取目标文件的属主
        mysqldcmd = '/proc/%s/exe' % var_mysql_pid
        mysqldcmd = os.readlink(mysqldcmd)        
        if os.path.exists(mysqldcmd) == False:
            raise Exception('File %s is not exists.' % mysqldcmd)
        if re.search(r';', mysqldcmd) or re.search(r'<', mysqldcmd) or re.search(r'>', mysqldcmd) or re.search(r'\\n', mysqldcmd):
            raise Exception('The mysqldcmd contains special characters ;<>\\n, which may lead to privilege escalation')
        self._runcontext.log_debug('mysqld = %s' % mysqldcmd)
        self._runcontext.set_extra_var('MYSQL_MYSQLD',mysqldcmd)
        
        st = os.stat(mysqldcmd)
        mysqlduid = st.st_uid
        mysqlowner = self.get_user_name(mysqlduid)
        self._runcontext.set_extra_var('MYSQL_MYSQLD_OWNER',mysqlowner)
        if _find_unsafe(mysqldcmd) is not None:
            raise Exception('cmd %s is not safe!' % mysqldcmd)
        if mysqlduid != 0:
            mysqldcmd = 'su - %s -s /bin/bash -c "%s --verbose --help | grep dir"' % (mysqlowner, mysqldcmd)
        else:
            mysqldcmd = '%s --verbose --help | grep dir' % mysqldcmd
        self._runcontext.log_debug('mysqld command = %s' % mysqldcmd)
            
        p = subprocess.Popen(mysqldcmd, env=os.environ, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
        self._runcontext.sleep()
        output = p.stdout.read()
        if PY3:
            output = output.decode('utf-8')
        
        self._runcontext.log_debug(output)
        
        pattern = re.compile(r'^basedir\s+(\S+)',re.MULTILINE)
        matchs = pattern.findall(output)
        if len(matchs) == 1:
            self._runcontext.set_extra_var('MYSQL_BASEDIR',self._runcontext.replace_inject_characters(matchs[0].rstrip(os.sep)))
        else:
            self._runcontext.set_extra_var('MYSQL_BASEDIR','')
            
        pattern = re.compile(r'^datadir\s+(\S+)',re.MULTILINE)
        matchs = pattern.findall(output)
        if len(matchs) == 1:
            self._runcontext.set_extra_var('MYSQL_DATADIR',self._runcontext.replace_inject_characters(matchs[0].rstrip(os.sep)))
        else:
            self._runcontext.set_extra_var('MYSQL_DATADIR','')
            
        pattern = re.compile(r'^plugin-dir\s+(\S+)',re.MULTILINE)
        matchs = pattern.findall(output)
        if len(matchs) == 1:
            self._runcontext.set_extra_var('MYSQL_PLUGINDIR',self._runcontext.replace_inject_characters(matchs[0].rstrip(os.sep)))
        else:
            self._runcontext.set_extra_var('MYSQL_PLUGINDIR','')       
        
        #ps -wp 84465 -o pid,ppid,user,cmd
        #get parent process information
        p = subprocess.Popen('ps -wp %s -o pid,ppid,user,cmd' % (var_mysql_pid), env=os.environ, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
        self._runcontext.sleep()
        output = p.stdout.read()
        if PY3:
            output = output.decode('utf-8')
        self._runcontext.log_debug(output)
        pattern = re.compile(r'%s\s+(\d+)\s+(\S+)\s+(.*)' % var_mysql_pid)
        matchs = pattern.findall(output)
        if len(matchs) != 1:
            raise Exception('Could not find process with pid %s' % var_mysql_pid)
            
        var_ppid = matchs[0][0]
        var_puser = matchs[0][1]
        var_pcmd = matchs[0][2]
        
        self._runcontext.log_debug('PID : %s' % var_mysql_pid)
        self._runcontext.log_debug('PUSER : %s' % var_puser)
        self._runcontext.log_debug('PCMD : %s' % var_pcmd)
        
        self._runcontext.set_extra_var('MYSQL_COMMAND',var_pcmd)
        
        p = subprocess.Popen('ps -wp %s -o pid,user,cmd' % (self._runcontext.replace_inject_characters(var_ppid)), env=os.environ, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
        self._runcontext.sleep()
        output = p.stdout.read()
        if PY3:
            output = output.decode('utf-8')
        self._runcontext.log_debug(output)
        pattern = re.compile(r'%s\s+(\S+)\s+(.*)' % var_ppid)
        matchs = pattern.findall(output)
        if len(matchs) != 1:
            raise Exception('Could not find process with pid %s' % var_ppid)

        var_ppuser = matchs[0][0]
        var_ppcmd = matchs[0][1]        
    
        pattern = re.compile(r'\s+--basedir=(.*?)(?:\s|$)')
        matchs = pattern.findall(var_pcmd)
        if len(matchs) != 1:
            pass
        else:
            var_basedir = matchs[0]
            self._runcontext.set_extra_var('MYSQL_BASEDIR', var_basedir)
        
        self._runcontext.log_debug('PPID : %s' % var_ppid)
        self._runcontext.log_debug('PPUSER : %s' % var_ppuser)
        self._runcontext.log_debug('PPCMD : %s' % var_ppcmd)
        
        if 'mysqld_multi' in var_ppcmd:
            self._runcontext.set_parameter('_mysql_conf_parse',None)
            return
        
        load_cnf_files = []

        if '--defaults-file' in var_pcmd:
            pattern = re.compile(r'--defaults-file=(\S+)')
            matchs = pattern.findall(var_pcmd)
            if len(matchs) < 1:
                raise Exception('Could not get defaults-file in command : %s' % var_pcmd) 

            load_cnf_files.append(matchs[0])
        else:
            if os.path.exists('/etc/my.cnf'):
                load_cnf_files.append('/etc/my.cnf')
            if os.path.exists('/etc/mysql/my.cnf'):
                load_cnf_files.append('/etc/mysql/my.cnf')
            var_mysql_home = self._runcontext.get_extra_var('MYSQL_BASEDIR')
            if os.path.exists(var_mysql_home)== True:
                t = os.path.realpath(os.path.join(var_mysql_home,'my.cnf'))
                if os.path.exists(t):
                    load_cnf_files.append(t)
            if '--defaults-extra-file' in var_pcmd:
                pattern = re.compile(r'--defaults-extra-file=(\S+)')
                matchs = pattern.findall(var_pcmd)
                if len(matchs) < 1:
                    raise Exception('Could not get defaults-extra-file in command : %s' % var_pcmd)
                load_cnf_files.append(matchs[0])
            for user in pwd.getpwall():
                if user.pw_name == var_ppuser:
                    t = os.path.join(user.pw_dir,'.my.cnf')
                    if os.path.exists(t):
                        load_cnf_files.append(t)
                        break
            
        parse = MySQLParse()
        for f in load_cnf_files:
            self._runcontext.log_debug(f)
            retRes, msg = parse.read_cnf(f)
            if not retRes:
                self._runcontext.log_error(msg)

        self._runcontext.set_parameter('_mysql_conf_parse',parse)
        self._runcontext.log_debug(parse.pprint())
        
        self._runcontext.set_parameter('cur_cnf_files',load_cnf_files)
        self._runcontext.set_extra_var('LOADED_CONF_FILES',load_cnf_files)
		
        mysqld = self._runcontext.get_extra_var('MYSQL_MYSQLD')
        if "root" != mysqlowner:
            mysqldcmd = 'su - %s -s /bin/bash -c "%s --version"' % (mysqlowner, mysqld)
        else:
            mysqldcmd = '%s --version' % mysqld
        p = subprocess.Popen(mysqldcmd, env=os.environ, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
        self._runcontext.sleep()
        output = p.stdout.read()
        if PY3:
            output = output.decode('utf-8')

        regex = re.compile(r'.*Ver\s*(\d+\.\d+\.\d+)')
        find_list = regex.findall(output)
        if find_list:
            self._runcontext.set_extra_var('MYSQL_VERSION', find_list[0])
        else:
            raise Exception("Get MySQL Version Failed !")

        self._runcontext.log_debug(output)

    def get_tasks(self, dir):
        ret = []
        ret.append([dir, 'r_2_1_4.py'])
        ret.append([dir, 's_2_1_8.py'])
        ret.append([dir, 's_2_1_9.py'])
        ret.append([dir, 'r_2_1_10.py'])
        ret.append([dir, 'r_2_1_11.py'])
        ret.append([dir, 'r_2_1_12.py'])
        ret.append([dir, 'r_2_1_13.py'])

        ret.append([dir, 'r_2_2_6.py'])
        ret.append([dir, 'r_2_2_7.py'])
        ret.append([dir, 'r_2_2_8.py'])
        ret.append([dir, 's_2_2_9.py'])
        ret.append([dir, 'r_2_2_12.py'])
        ret.append([dir, 'r_2_2_13.py'])
        ret.append([dir, 'r_2_2_14.py'])
        ret.append([dir, 'r_2_2_15.py'])
        ret.append([dir, 'r_2_2_16.py'])
        ret.append([dir, 'r_2_2_17.py'])
        ret.append([dir, 'r_2_2_18.py'])

        ret.append([dir, 'r_2_4_1.py'])
        ret.append([dir, 'r_2_4_2.py'])
        ret.append([dir, 'r_2_4_7.py'])

        ret.append([dir, 'r_3_1_1.py'])
        ret.append([dir, 'r_3_1_2.py'])
        
        ret.append([dir, 'r_3_2_3.py'])
        ret.append([dir, 'r_3_2_4.py'])
        ret.append([dir, 'r_3_2_5.py'])
        ret.append([dir, 'r_3_2_6.py'])
        ret.append([dir, 'r_3_2_7.py'])
        ret.append([dir, 'r_3_2_8.py'])

        return ret
    
if __name__ == '__main__':
    
    myplaybook = MyPlaybook()
    myplaybook.set_role('hw')
    myplaybook.run()
    

