﻿# -*- coding: UTF-8 -*-
import os
import re
import json


def get_tags():
    return ['3', '3.2', '3.2.5']


def run_task(runcontext):
    ansibleParams = runcontext.get_mod_params_instance()
    runcontext.set_mod_params_instance(ansibleParams, "chdir", None)
    runcontext.set_mod_params_instance(ansibleParams, "executable", None)
    module = runcontext.get_module_file("shell.py")
    rc, out = run_audit(runcontext, ansibleParams, module)
    return rc, out


def run_audit(runcontext, ansibleParams, module):
    curtemplate = runcontext.get_template()
    tmpVars = {}
    tmpVars["policyresult"] = "failed"
    tmpVars["checkresult1"] = "pass"
    tmpVars["checkresult2"] = "pass"
    tmpVars["checkresult3"] = "pass"
    tmpVars["checkresult4"] = "pass"
    tmpVars["checkresult5"] = "pass"
    tmpVars["scancommand1"] = ''
    tmpVars["scancommand2"] = ''
    tmpVars["scancommand3"] = ''
    tmpVars["scancommand4"] = ''
    tmpVars["scancommand5"] = ''
    tmpVars["systemvalue1"] = ''
    tmpVars["systemvalue2"] = ''
    tmpVars["systemvalue3"] = ''
    tmpVars["systemvalue4"] = ''
    tmpVars["systemvalue5"] = ''
    retCode = -1

    host_name = "echo $HOSTNAME"
    runcontext.set_mod_params_instance(ansibleParams, "command", host_name)
    arc, astdout, astderr = runcontext.invoke_module(module, ansibleParams)
    runcontext.log_debug_invoke_result(arc, astdout, astderr)
    runcontext.sleep()
    if arc != 0:
        return -2, "Failed to invoke module: %s." % astdout
    resultjson = json.loads(astdout)
    astdout = resultjson["stdout"]
    runcontext.log_debug_json(resultjson)

    var_mysql_parser = runcontext.get_parameter('_mysql_conf_parse')
    var_mysql_cmd = runcontext.get_extra_var('MYSQL_COMMAND')

    # get datadir
    mysql_basedir = runcontext.get_extra_var('MYSQL_BASEDIR')
    if var_mysql_parser is not None:
        tmp = var_mysql_parser.get_option('mysqld', 'basedir')
        runcontext.log_debug("tmp1: %s" % tmp)
        if tmp is not None:
            mysql_basedir = tmp['value']

    pattern = re.compile(r'(--basedir)(=)(\S+)')
    matchs = pattern.findall(var_mysql_cmd)
    if len(matchs) > 0:
        mysql_basedir = matchs[0][2]
    mysql_basedir = runcontext.replace_inject_characters(mysql_basedir)

    # get datadir
    mysql_datadir = runcontext.get_extra_var('MYSQL_DATADIR')
    var_mysql_parser = runcontext.get_parameter('_mysql_conf_parse')
    if var_mysql_parser is not None:
        tmp = var_mysql_parser.get_option('mysqld', 'datadir')
        runcontext.log_debug("tmp2: %s" % tmp)
        if tmp is not None:
            mysql_datadir = tmp['value']

    pattern = re.compile(r'(--datadir)(=)(\S+)')
    matchs = pattern.findall(var_mysql_cmd)
    if len(matchs) > 0:
        mysql_datadir = matchs[0][2]
    mysql_datadir = runcontext.replace_inject_characters(mysql_datadir)

    # find log-error
    cur_log_error_value = ""
    if var_mysql_parser is not None:
        tmp = var_mysql_parser.get_option('mysqld', 'log_error')
        runcontext.log_debug("tmp3: %s" % tmp)
        if tmp is not None:
            if tmp['value'] == "":
                cur_log_error_value = os.path.join(mysql_datadir, "%s.err" % astdout)
            elif os.path.exists(tmp['value']):
                cur_log_error_value = tmp['value']
            else:
                cur_log_error_value = os.path.join(mysql_datadir, "%s.err" % tmp['value'])

    pattern = re.compile(r'--log-error(\S+)')
    matchs = pattern.findall(var_mysql_cmd)
    if "--log-error" in var_mysql_cmd:
        if len(matchs) == 0:
            cur_log_error_value = os.path.join(mysql_datadir, "%s.err" % astdout)
        else:
            match = matchs[-1].split("=")[-1]
            if os.path.exists(match):
                cur_log_error_value = match
            else:
                cur_log_error_value = os.path.join(mysql_datadir, "%s.err" % match)

    # find slow_query_log
    cur_slow_query_log_file_value = ""
    if var_mysql_parser is not None:
        tmp = var_mysql_parser.get_option('mysqld', 'slow_query_log')
        tmp2 = var_mysql_parser.get_option('mysqld', 'slow_query_log_file')
        runcontext.log_debug("tmp4: %s" % tmp)
        runcontext.log_debug("tmp5: %s" % tmp2)
        if tmp is not None:
            if tmp['key'] == "slow_query_log" or tmp['key'] == "slow-query-log" or tmp['value'] == "ON" or tmp['value'] == "TRUE" or tmp['value'] == 1:
                if tmp2 is not None:
                    if tmp2['value'] != "":
                        if os.path.exists(tmp2['value']):
                            cur_slow_query_log_file_value = tmp2['value']
                        else:
                            cur_slow_query_log_file_value = os.path.join(mysql_datadir, tmp2['value'])
                else:
                    cur_slow_query_log_file_value = os.path.join(mysql_datadir, "%s-slow.log" % astdout)

    pattern = re.compile(r'--slow-query-log(=\S+)')
    pattern2 = re.compile(r'--slow-query-log-file=(\S+)')
    matchs = pattern.findall(var_mysql_cmd)
    matchs2 = pattern2.findall(var_mysql_cmd)
    if "--slow-query-log" in var_mysql_cmd:
        if len(matchs) == 0:
            if len(matchs2) > 0:
                if os.path.exists(matchs2[-1]):
                    cur_slow_query_log_file_value = matchs2[-1]
                else:
                    cur_slow_query_log_file_value = os.path.join(mysql_datadir, matchs2[-1])
            else:
                cur_slow_query_log_file_value = os.path.join(mysql_datadir, "%s-slow.log" % astdout)
        else:
            match = matchs[-1].split("=")[-1]
            if match == "ON" or match == "TRUE" or match == 1:
                if len(matchs2) > 0:
                    if os.path.exists(matchs2[-1]):
                        cur_slow_query_log_file_value = matchs2[-1]
                    else:
                        cur_slow_query_log_file_value = os.path.join(mysql_datadir, matchs2[-1])
                else:
                    cur_slow_query_log_file_value = os.path.join(mysql_datadir, "%s-slow.log" % astdout)

    # find general_log_file
    cur_general_log_value = ""
    if var_mysql_parser is not None:
        tmp = var_mysql_parser.get_option('mysqld', 'general_log')
        tmp2 = var_mysql_parser.get_option('mysqld', 'general_log_file')
        runcontext.log_debug("tmp6: %s" % tmp)
        runcontext.log_debug("tmp7: %s" % tmp2)
        if tmp is not None:
            if tmp['key'] == "general_log" or tmp['key'] == "general-log" or tmp['value'] == "ON" or tmp['value'] == "TRUE" or tmp['value'] == 1:
                if tmp2 is not None:
                    if tmp2['value'] != "":
                        if os.path.exists(tmp2['value']):
                            cur_general_log_value = tmp2['value']
                        else:
                            cur_general_log_value = os.path.join(mysql_datadir, tmp2['value'])
                else:
                    cur_general_log_value = os.path.join(mysql_datadir, "%s.log" % astdout)

    pattern = re.compile(r'--general-log(=\S+)')
    pattern2 = re.compile(r'--general-log-file=(\S+)')
    matchs = pattern.findall(var_mysql_cmd)
    matchs2 = pattern2.findall(var_mysql_cmd)
    if "--general-log" in var_mysql_cmd:
        if len(matchs) == 0:
            if len(matchs2) > 0:
                if os.path.exists(matchs2[-1]):
                    cur_general_log_value = matchs2[-1]
                else:
                    cur_general_log_value = os.path.join(mysql_datadir, matchs2[-1])
            else:
                cur_general_log_value = os.path.join(mysql_datadir, "%s.log" % astdout)
        else:
            match = matchs[-1].split("=")[-1]
            if match == "ON" or match == "TRUE" or match == 1:
                if len(matchs2) > 0:
                    if os.path.exists(matchs2[-1]):
                        cur_general_log_value = matchs2[-1]
                    else:
                        cur_general_log_value = os.path.join(mysql_datadir, matchs2[-1])
                else:
                    cur_general_log_value = os.path.join(mysql_datadir, "%s.log" % astdout)

    # find log_bin_basename
    cur_log_bin_basename_value = ""
    if var_mysql_parser is not None:
        tmp = var_mysql_parser.get_option('mysqld', 'log_bin')
        runcontext.log_debug("tmp8: %s" % tmp)
        if tmp is not None:
            if tmp['value'] == "":
                cur_log_bin_basename_value = os.path.join(mysql_datadir, "%s-bin*" % astdout)
            else:
                cur_log_bin_basename_value = os.path.join(mysql_datadir, "%s*" % tmp['value'])

    pattern = re.compile(r'--log-bin(\S+)')
    matchs = pattern.findall(var_mysql_cmd)
    if "--log-bin" in var_mysql_cmd:
        if len(matchs) == 0:
            cur_log_bin_basename_value = os.path.join(mysql_datadir, "%s-bin*" % astdout)
        else:
            match = matchs[-1].split("=")[-1]
            cur_log_bin_basename_value = os.path.join(mysql_datadir, "%s*" % match)

    # find relay_log_basename
    cur_relay_log_basename_value = ""
    if var_mysql_parser is not None:
        tmp = var_mysql_parser.get_option('mysqld', 'relay_log')
        runcontext.log_debug("tmp9: %s" % tmp)
        if tmp is not None:
            if tmp['value'] != "":
                cur_relay_log_basename_value = os.path.join(mysql_datadir, "%s*" % tmp['value'])

    pattern = re.compile(r'--relay-log=(\S+)')
    matchs = pattern.findall(var_mysql_cmd)
    if len(matchs) > 0:
        cur_relay_log_basename_value = os.path.join(mysql_datadir, "%s*" % matchs[-1])

    cur_log_error_value = runcontext.replace_inject_characters(cur_log_error_value)
    if cur_log_error_value != '':
        tmpVars["scancommand1"] = "access=$(stat -L --format=\"%s\" %s);user=$(stat -L --format=\"%s\" %s);group=$(stat -L --format=\"%s\" %s);if (((0$access & 0177) != 0 )) || [ \"$user\" == 'root' ] || [ \"$group\" == 'root' ];then echo \"%s $user:$group $access \";else echo \"%s $user:$group $access\";fi" % ('%a', cur_log_error_value, '%U', cur_log_error_value, '%G', cur_log_error_value, cur_log_error_value, cur_log_error_value)
        runcontext.set_mod_params_instance(ansibleParams, "command", tmpVars["scancommand1"])
        arc1, astdout1, astderr1 = runcontext.invoke_module(module, ansibleParams)
        runcontext.log_debug_invoke_result(arc1, astdout1, astderr1)
        runcontext.sleep()
        if arc1 != 0:
            return (-2, "Failed to invoke module: %s." % astdout1)

        resultjson1 = json.loads(astdout1)
        astdout1 = resultjson1["stdout"]
        runcontext.log_debug_json(resultjson1)
        match1_found = re.match(r".+\s$", astdout1)
        if match1_found:
            tmpVars["systemvalue1"] = astdout1
            tmpVars["checkresult1"] = "failed"
        else:
            tmpVars["systemvalue1"] = astdout1

    cur_slow_query_log_file_value = runcontext.replace_inject_characters(cur_slow_query_log_file_value)
    if cur_slow_query_log_file_value != '':
        tmpVars["scancommand2"] = "access=$(stat -L --format=\"%s\" %s);user=$(stat -L --format=\"%s\" %s);group=$(stat -L --format=\"%s\" %s);if (((0$access & 0177) != 0 )) || [ \"$user\" == 'root' ] || [ \"$group\" == 'root' ];then echo \"%s $user:$group $access \";else echo \"%s $user:$group $access\";fi" % ('%a', cur_slow_query_log_file_value, '%U', cur_slow_query_log_file_value, '%G', cur_slow_query_log_file_value, cur_slow_query_log_file_value, cur_slow_query_log_file_value)
        runcontext.set_mod_params_instance(ansibleParams, "command", tmpVars["scancommand2"])
        arc2, astdout2, astderr2 = runcontext.invoke_module(module, ansibleParams)
        runcontext.log_debug_invoke_result(arc2, astdout2, astderr2)
        runcontext.sleep()
        if arc2 != 0:
            return (-2, "Failed to invoke module: %s." % astdout2)

        resultjson2 = json.loads(astdout2)
        astdout2 = resultjson2["stdout"]
        runcontext.log_debug_json(resultjson2)
        match2_found = re.match(r".+\s$", astdout2)
        if match2_found:
            tmpVars["systemvalue2"] = astdout2
            tmpVars["checkresult2"] = "failed"
        else:
            tmpVars["systemvalue2"] = astdout2

    cur_general_log_value = runcontext.replace_inject_characters(cur_general_log_value)
    if cur_general_log_value != '':
        tmpVars["scancommand3"] = "access=$(stat -L --format=\"%s\" %s);user=$(stat -L --format=\"%s\" %s);group=$(stat -L --format=\"%s\" %s);if (((0$access & 0177) != 0 )) || [ \"$user\" == 'root' ] || [ \"$group\" == 'root' ];then echo \"%s $user:$group $access \";else echo \"%s $user:$group $access\";fi" % ('%a', cur_general_log_value, '%U', cur_general_log_value, '%G', cur_general_log_value, cur_general_log_value, cur_general_log_value)
        runcontext.set_mod_params_instance(ansibleParams, "command", tmpVars["scancommand3"])
        arc3, astdout3, astderr3 = runcontext.invoke_module(module, ansibleParams)
        runcontext.log_debug_invoke_result(arc3, astdout3, astderr3)
        runcontext.sleep()
        if arc3 != 0:
            return (-2, "Failed to invoke module: %s." % astdout3)

        resultjson3 = json.loads(astdout3)
        astdout3 = resultjson3["stdout"]
        runcontext.log_debug_json(resultjson3)
        match3_found = re.match(r".+\s$", astdout3)
        if match3_found:
            tmpVars["systemvalue3"] = astdout3
            tmpVars["checkresult3"] = "failed"
        else:
            tmpVars["systemvalue3"] = astdout3

    cur_log_bin_basename_value = runcontext.replace_inject_characters(cur_log_bin_basename_value)
    if cur_log_bin_basename_value != '':
        tmpVars["scancommand4"] = '''stat -L --format="%s %s %s %s" %s''' % ('%n', '%U','%G', '%a', cur_log_bin_basename_value)
        runcontext.set_mod_params_instance(ansibleParams, "command", tmpVars["scancommand4"])
        arc4, astdout4, astderr4 = runcontext.invoke_module(module, ansibleParams)
        runcontext.log_debug_invoke_result(arc4, astdout4, astderr4)
        runcontext.sleep()
        if arc4 != 0:
            return (-2, "Failed to invoke module: %s." % astdout4)

        resultjson4 = json.loads(astdout4)
        astdout4 = resultjson4["stdout"]
        runcontext.log_debug_json(resultjson4)
        match4_found = re.search(r"root|(6[1-7][0-7]|6[0-7][1-7]|7[0-7][0-7])", astdout4, re.M)
        if match4_found:
            tmpVars["systemvalue4"] = astdout4
            tmpVars["checkresult4"] = "failed"
        else:
            tmpVars["systemvalue4"] = astdout4

    cur_relay_log_basename_value = runcontext.replace_inject_characters(cur_relay_log_basename_value)
    if cur_relay_log_basename_value != '':
        tmpVars["scancommand5"] = '''stat -L --format="%s %s %s %s" %s''' % ('%n', '%U','%G', '%a', cur_relay_log_basename_value)
        runcontext.set_mod_params_instance(ansibleParams, "command", tmpVars["scancommand5"])
        arc5, astdout5, astderr5 = runcontext.invoke_module(module, ansibleParams)
        runcontext.log_debug_invoke_result(arc5, astdout5, astderr5)
        runcontext.sleep()
        if arc5 != 0:
            return (-2, "Failed to invoke module: %s." % astdout5)

        resultjson5 = json.loads(astdout5)
        astdout5 = resultjson5["stdout"]
        runcontext.log_debug_json(resultjson5)
        match5_found = re.search(r"root|(6[1-7][0-7]|6[0-7][1-7]|7[0-7][0-7])", astdout5, re.M)
        if match5_found:
            tmpVars["systemvalue5"] = astdout5
            tmpVars["checkresult5"] = "failed"
        else:
            tmpVars["systemvalue5"] = astdout5
    if tmpVars["checkresult1"] == "pass" and tmpVars["checkresult2"] == "pass" and tmpVars["checkresult3"] == "pass" and tmpVars["checkresult4"] == "pass" and tmpVars["checkresult5"] == "pass":
        tmpVars["policyresult"] = "pass"
        retCode = 0

    rc, out = curtemplate.render(runcontext.get_template_file('r_3_2_5.j2'), **tmpVars)

    if rc == False:
        return (-2, "Failed to render template.%s" % out)
    return retCode, out
