﻿# -*- coding: UTF-8 -*-
import os
import re
import json

def get_tags():
    return ['3','3.2','3.2.4']

def run_task(runcontext):
    ansible_params=runcontext.get_mod_params_instance()
    runcontext.set_mod_params_instance(ansible_params,"chdir",None)
    runcontext.set_mod_params_instance(ansible_params,"executable",None)
    module=runcontext.get_module_file("shell.py")
    rc,out=run_audit(runcontext,ansible_params,module)
    return rc,out

def run_audit(runcontext,ansible_params,module):
    curtemplate=runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["checkresult2"] = "failed"
    
    tmp_vars["systemvalue1"] = ''
    tmp_vars["systemvalue2"] = ''
    ret_code = -1
    
    cur_datadir = runcontext.get_extra_var('MYSQL_DATADIR')
    
    var_mysql_parser = runcontext.get_parameter('_mysql_conf_parse')
    if var_mysql_parser != None:
        tmp = var_mysql_parser.get_option('mysqld','datadir')
        runcontext.log_debug("tmp: %s" % tmp)
        if tmp != None:
            cur_datadir = tmp['value']

    var_mysql_cmd = runcontext.get_extra_var('MYSQL_COMMAND')
    pattern = re.compile(r'--datadir=(\S+)')
    matchs = pattern.findall(var_mysql_cmd)

    if len(matchs) > 0:
        cur_datadir = matchs[-1]
        
    cur_datadir = runcontext.replace_inject_characters(cur_datadir)
    
    if cur_datadir != '':
        if not os.path.isabs(cur_datadir):
            # get --basedir
            mysql_basedir = runcontext.get_extra_var('MYSQL_BASEDIR')
            var_mysql_parser = runcontext.get_parameter('_mysql_conf_parse')
            if var_mysql_parser != None:
                tmp = var_mysql_parser.get_option('mysqld','basedir')
                runcontext.log_debug("tmp2: %s" % tmp)
                if tmp != None:
                    mysql_basedir = tmp['value']
                    
            var_mysql_cmd = runcontext.get_extra_var('MYSQL_COMMAND')
            pattern = re.compile(r'(--basedir)(=)(\S+)')
            matchs = pattern.findall(var_mysql_cmd)
            
            if len(matchs) > 0:
                mysql_basedir = matchs[0][2]
            mysql_basedir = runcontext.replace_inject_characters(mysql_basedir)
            
            cur_datadir = os.path.join(mysql_basedir,cur_datadir)
  
        tmp_vars["scancommand1"] = "access=$(stat -L --format=\"%s\" %s);user=$(stat -L --format=\"%s\" %s);group=$(stat -L --format=\"%s\" %s);if (((0$access & 0077) != 0 )) || [ \"$user\" == 'root' ] || [ \"$group\" == 'root' ];then echo \"%s $user:$group $access\";fi" % ('%a',cur_datadir,'%U',cur_datadir,'%G',cur_datadir,cur_datadir)
        runcontext.set_mod_params_instance(ansible_params,"command",tmp_vars["scancommand1"])
        arc1,astdout1,astderr1=runcontext.invoke_module(module,ansible_params)
        runcontext.log_debug_invoke_result(arc1,astdout1,astderr1)
        runcontext.sleep()
        if arc1 != 0:
            return (-2, "Failed to invoke module: %s."%astdout1) 

        resultjson1 = json.loads(astdout1)
        astdout1 = resultjson1["stdout"]
        runcontext.log_debug_json(resultjson1)
        match1_found=re.match("^$",astdout1)
        if match1_found == None:
            tmp_vars["systemvalue1"] = astdout1
        else:
            tmp_vars["checkresult1"] = "pass"
            tmp_vars["systemvalue1"] = astdout1

            
        tmp_vars["scancommand2"] = "count=0;files=$(ls -ld %s/* | awk '{print $9}');for file in $files;do if [[ -f $file ]];then access=$(stat -L --format=%s $file);user=$(stat -L --format=%s $file);group=$(stat -L --format=%s $file);if (((0$access & 0177) != 0 )) || [ \"$user\" == 'root' ] || [ \"$group\" == 'root' ];then count=$[count+1];if [ \"$count\" -gt \"100\" ]; then break;else echo \"$file $user:$group $access\";fi;fi;fi;done;if [ \"$count\" -gt \"100\" ]; then echo \"Message length is more than 100!For more infomation manually execute the check policy.\";fi;" % (cur_datadir,'%a','%U','%G')
        runcontext.set_mod_params_instance(ansible_params,"command",tmp_vars["scancommand2"])
        arc2,astdout2,astderr2=runcontext.invoke_module(module,ansible_params)
        runcontext.log_debug_invoke_result(arc2,astdout2,astderr2)
        runcontext.sleep()
        if arc2 != 0:
            return (-2, "Failed to invoke module: %s."%astdout2) 

        resultjson2 = json.loads(astdout2)
        astdout2 = resultjson2["stdout"]
        runcontext.log_debug_json(resultjson2)
        match2_found=re.match("^$",astdout2)
        if match2_found == None:
            tmp_vars["systemvalue2"] = astdout2
        else:
            tmp_vars["checkresult2"] = "pass"
            tmp_vars["systemvalue2"] = astdout2

    if tmp_vars["checkresult1"] == 'pass' and tmp_vars["checkresult2"] == 'pass':
        tmp_vars["policyresult"]="pass"
        ret_code = 0


    rc,out = curtemplate.render(runcontext.get_template_file('r_3_2_4.j2'),**tmp_vars)

    if rc == False:
        return (-2,"Failed to render template.%s" % out)
    return ret_code,out
